@@ -360,6 +360,18 @@ class BpApi(): # pylint: disable=too-many-instance-attributes,too-many-public-me
     eprint('Analysis {} has been restarted'.format(uid))
     return True
 
+  def terminate_analysis(self, uid):
+    '''Terminate analysis'''
+    payload = {
+      'id': uid,
+      'source': 'cli',
+    }
+    res = Analysis(self.conf.get('api')).terminate(payload=payload)
+    if res.get('error'):
+      return False
+    eprint('INFO: Termination process initiated for analysis id - {}.'.format(uid))
+    return True
+
   def update_analysis(self, uid, data):
     '''Update analysis'''
     info = (Analysis(self.conf.get('api'))).save(
@@ -607,27 +619,34 @@ class BpApi(): # pylint: disable=too-many-instance-attributes,too-many-public-me
     '''Get pipelines list'''
     return Pipeline(self.conf.get('api')).list_all(filters=filters)
 
-  def update_pipeline(self,data):
+  def update_pipeline(self, data=None, params=None, pipeline_id=None):
     '''update pipeline from yaml'''
-    try:
-      path = os.path.abspath(os.path.expanduser(os.path.expandvars(data['yamlpath'])))
-      with open(path, 'r') as file:
-        yaml_string = file.read()
-      yaml_data = yaml.load(yaml_string, Loader=yaml.FullLoader)
-      workflow_id = yaml_data.get('id')
-      if not yaml_data.get('name'):
-        sys.exit('Please provide pipeline name in YAML')
-      if not yaml_data.get('id'):
-        sys.exit('Please provide pipeline id in YAML')
-      payload = {'data': yaml_string}
-      info = (Pipeline(self.conf.get('api'))).save(obj_id=workflow_id, payload=payload)
+    if data:
+      try:
+        path = os.path.abspath(os.path.expanduser(os.path.expandvars(data['yamlpath'])))
+        with open(path, 'r') as file:
+          yaml_string = file.read()
+        yaml_data = yaml.load(yaml_string, Loader=yaml.FullLoader)
+        workflow_id = yaml_data.get('id')
+        if not yaml_data.get('name'):
+          sys.exit('Please provide pipeline name in YAML')
+        if not yaml_data.get('id'):
+          sys.exit('Please provide pipeline id in YAML')
+        payload = {'data': yaml_string}
+        info = (Pipeline(self.conf.get('api'))).save(obj_id=workflow_id, payload=payload)
+        if info.get('error'):
+          sys.exit('ERROR: failed while updating pipeline')
+        if info.get('id'):
+          eprint('updated: pipeline {} with id {}'.format(info.get('name'), info.get('id')))
+          return
+      except Exception:# pylint: disable=bare-except
+        sys.exit('ERROR: Something went wrong while updating pipeline.')
+    if params:
+      info = (Pipeline(self.conf.get('api'))).save(obj_id=pipeline_id, params=params)
       if info.get('error'):
         sys.exit('ERROR: failed while updating pipeline')
       if info.get('id'):
         eprint('updated: pipeline {} with id {}'.format(info.get('name'), info.get('id')))
-        return
-    except Exception:# pylint: disable=bare-except
-      sys.exit('ERROR: Something went wrong while updating pipeline.')
 
   ################################################################################################
   ### PROJECT ####################################################################################
@@ -696,9 +715,6 @@ class BpApi(): # pylint: disable=too-many-instance-attributes,too-many-public-me
     if data.get('filepaths2') and isinstance(data['filepaths2'], str):
       data['filepaths2'] = [data['filepaths2']]
 
-    if data.get('filepaths2') and not data.get('filepaths1'):
-      sys.exit('ERROR: Parameter filepaths1 cannot be empty if filepaths2 is specified.')
-
     # validate unique file between filepaths1 and filepaths2
     does_repeat = data.get('filepaths2') \
       and any(path in data.get('filepaths2') for path in data.get('filepaths1'))
@@ -881,7 +897,7 @@ class BpApi(): # pylint: disable=too-many-instance-attributes,too-many-public-me
 
   def upload_uri_to_id(self, uri):
     '''Get upload from uri and return upload id'''
-    response = (Upload(self.conf.get('api'))).list({'limit': 2, 'uri': uri})
+    response = (Upload(self.conf.get('api'))).list(params={'limit': 2, 'uri': uri})
     items = response.get('objects')
     if not items:
       eprint('WARNING: no upload by uri', uri)
@@ -953,13 +969,14 @@ class BpApi(): # pylint: disable=too-many-instance-attributes,too-many-public-me
     try:
       # get the download directory
       prefix = self.scratch
+      suffix = 'basepair/'      
       if file_type == 'analyses':
         analyses_type_path = os.path.dirname(filekey)
         analyses_type = os.path.basename(analyses_type_path)
       if dirname:
         prefix = dirname if dirname.startswith('/') else os.path.join(self.scratch, dirname)
-      suffix = 'basepair/'
-      if file_type == 'analyses' and uid:
+        suffix = ''
+      elif file_type == 'analyses' and uid:
         suffix = 'basepair/{}/{}/{}'.format(file_type, uid, analyses_type)
       elif file_type and uid:
         suffix = 'basepair/{}/{}'.format(file_type, uid)
@@ -1006,8 +1023,11 @@ class BpApi(): # pylint: disable=too-many-instance-attributes,too-many-public-me
       if not files:
         eprint('Warning: No files present for sample with id {}'.format(uid))
         return False
+      results = []
       for file_i in files:
-        return self.download_file(file_i, file_type=file_type, uid=uid, dirname=outdir)
+        result = self.download_file(file_i, file_type=file_type, uid=uid, dirname=outdir)
+        results.append(result)
+      return all(results)
     except Exception:# pylint: disable=broad-except
       return False
 
@@ -1136,6 +1156,12 @@ class BpApi(): # pylint: disable=too-many-instance-attributes,too-many-public-me
         _params += ' {} "{}"'.format(arg, val)
 
     storage_cfg = self.configuration.get_user_storage()
+    # Check for local storage params present in storage_cfg
+    local_storage_params = {'endpoint_url': '--endpoint-url', 'region': '--region'}
+    for param, flag in local_storage_params.items():
+      if storage_cfg.get(param):
+        _params += ' {} {}'.format(flag, storage_cfg.get(param))
+
     credential = self.configuration.get_cli_credentials_from(storage_cfg)
     return '{}aws s3 cp "{}" "{}" {}'.format(credential, src, dest, _params)
 
@@ -1266,10 +1292,7 @@ class BpApi(): # pylint: disable=too-many-instance-attributes,too-many-public-me
         if should_continue:
           continue
 
-        if analysis['params'] and 'info' in analysis['params']:
-          if not analysis['params']['info'].get('genome_id'):
-            eprint('Could not find genome for analysis {}'.format(analysis['id']))
-            continue
+        if analysis['params'] and 'info' in analysis['params'] and 'genome_id' in analysis['params']['info']:
           sample_genome_id = self.get_id_from_url(sample.get('genome', ''))
           if not sample_genome_id:
             eprint('Could not find genome for analysis {}'.format(analysis['id']))
