@@ -6,5 +6,5 @@ from .abstract import Abstract
 class File(Abstract):
   '''Webapp File class'''
   def __init__(self, cfg):
-    super().__init__(cfg)
+    super(File, self).__init__(cfg)
     self.endpoint += 'files/'
